--- old/usr/src/lib/libc/port/locale/lmonetary.c	2013-06-16 17:44:33.689376408 +0000
+++ new/usr/src/lib/libc/port/locale/lmonetary.c	2013-06-16 17:44:33.706636769 +0000
@@ -3,6 +3,11 @@
  * Copyright (c) 2000, 2001 Alexey Zelkin <phantom@FreeBSD.org>
  * All rights reserved.
  *
+ * Copyright (c) 2011 The FreeBSD Foundation
+ * All rights reserved.
+ * Portions of this software were developed by David Chisnall
+ * under sponsorship from the FreeBSD Foundation.
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -29,6 +34,7 @@
 #include <limits.h>
 #include <stddef.h>
 #include <stdlib.h>
+
 #include "ldpart.h"
 #include "lmonetary.h"
 
@@ -70,6 +76,8 @@
 static int	_monetary_using_locale;
 static char	*_monetary_locale_buf;
 
+struct xlocale_monetary __xlocale_global_monetary;
+
 static char
 cnv(const char *str)
 {
@@ -80,8 +88,20 @@
 	return ((char)i);
 }
 
-int
-__monetary_load_locale(const char *name)
+static void
+destruct_monetary(void *v)
+{
+	struct xlocale_monetary *l = v;
+
+	if (l->buffer != NULL)
+		free(l->buffer);
+
+	free(l);
+}
+
+static int
+monetary_load_locale_l(struct xlocale_monetary *loc, int *using_locale,
+    int *changed, const char *name)
 {
 	int ret;
 
@@ -130,9 +150,35 @@
 	return (ret);
 }
 
+int
+__monetary_load_locale(const char *name)
+{
+	return (monetary_load_locale_l(&__xlocale_global_monetary,
+	    &__xlocale_global_locale.using_monetary_locale,
+	    &__xlocale_global_locale.monetary_locale_changed, name));
+}
+
+void *
+__monetary_load(const char *name, locale_t loc)
+{
+	struct xlocale_monetary *new;
+
+	new = calloc(sizeof(struct xlocale_monetary), 1);
+	/* XXX */
+	new->header.header.destructor = destruct_monetary;
+	if (monetary_load_locale_l(new, &loc->using_monetary_locale,
+	    &loc->monetary_locale_changed, name) == _LDP_ERROR) {
+		xlocale_release(new);
+		return (NULL);
+	}
+
+	return (NULL);
+}
+
 struct lc_monetary_T *
-__get_current_monetary_locale(void)
+__get_current_monetary_locale(locale_t loc)
 {
-	return (_monetary_using_locale ? &_monetary_locale :
-	    (struct lc_monetary_T *)&_C_monetary_locale);
+	return (loc->using_monetary_locale
+	    ? &((struct xlocale_monetary*)loc->components[XLC_MONETARY])->locale
+	    : (struct lc_monetary_T *)&_C_monetary_locale);
 }
